﻿/*-------------------------------------------------------------------------
 SASEBO localbus descriptor
 
 File name   : SASEBO_LocalBus.cs
 Version     : Version 1.1
 Created     : Aug/20/2008
 Last update : Aug/18/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008, 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Text;


namespace SASEBO
{
  //================================================================ LocalBus
  public class LocalBus {
    //************************************************ Variable
    private volatile ControlInterface ctrlif;
    public static int MAX_BUF = 8192;
    private volatile byte[] buf = new byte[MAX_BUF];
    
		//************************************************ Method
    //------------------------------------------------ Constructor
    public LocalBus(ControlInterface ctrlif) {
      this.ctrlif = ctrlif;
      this.ctrlif.open();
    }

    //------------------------------------------------ Destructor
    ~LocalBus() {
      ctrlif.close();
    }
    
    //------------------------------------------------ write()
    public void write(int addr, int dat) {
      buf[0] = (byte)0x01;
      buf[1] = (byte)((addr>>8)&0xFF);
      buf[2] = (byte)((addr   )&0xFF);
      buf[3] = (byte)((dat >>8)&0xFF);
      buf[4] = (byte)((dat    )&0xFF);
      ctrlif.write(buf, 5);
    }
  
    //------------------------------------------------ read()
    public int read (int addr) {
      buf[0] = (byte)0x00;
      buf[1] = (byte)((addr>>8)&0xFF);
      buf[2] = (byte)((addr   )&0xFF);
      ctrlif.write(buf, 3);
      ctrlif.read (buf, 2);
      return ((int)buf[0]<<8) + (int)buf[1];
    }

    //------------------------------------------------ writeBurst()
    public void writeBurst(int addr, byte[] dat, int len) {
      for (int i=0; i<len/2; i++) {
        buf[i*5+0] = (byte)0x01;
        buf[i*5+1] = (byte)(((addr+i*2) >> 8) & 0xFF);
        buf[i*5+2] = (byte)(((addr+i*2)     ) & 0xFF);
        buf[i*5+3] = dat[i*2];
        buf[i*5+4] = dat[i*2+1];
      }
      ctrlif.write(buf, 5*(len/2));
    }

    //------------------------------------------------ readBurst()
    public void readBurst(int addr, byte[] dat, int len) {
      for (int i=0; i<len/2; i++) {
        buf[i*3+0] = (byte)0x00;
        buf[i*3+1] = (byte)(((addr+i*2) >> 8) & 0xFF);
        buf[i*3+2] = (byte)(((addr+i*2)     ) & 0xFF);
      }
      ctrlif.write(buf, 3*(len/2));
      ctrlif.read (dat, len);
    }

    //------------------------------------------------ close()
    public void close()
    {
      ctrlif.close();
    }
  }
}
