﻿/*-------------------------------------------------------------------------
 Cipher algorithm interface
 
 File name   : Cipher.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : Sep/03/2008
 Desgined by : Takeshi Sugawara
 Modified by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST and Tohoku Univ.
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST and Tohoku University ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace Cipher {
	//================================================================ DES
	public class DES : CipherInterface
	{
		//************************************************ Variables
		#region Declaration of tables
		static private readonly Byte[] S1 = new Byte[64] {
            14,4,13,1,2,15,11,8,3,10,6,12,5,9,0,7,
            0,15,7,4,14,2,13,1,10,6,12,11,9,5,3,8,
            4,1,14,8,13,6,2,11,15,12,9,7,3,10,5,0,
            15,12,8,2,4,9,1,7,5,11,3,14,10,0,6,13};

		static private readonly Byte[] S2 = new Byte[64] {
            15,1,8,14,6,11,3,4,9,7,2,13,12,0,5,10,
            3,13,4,7,15,2,8,14,12,0,1,10,6,9,11,5,
            0,14,7,11,10,4,13,1,5,8,12,6,9,3,2,15,
            13,8,10,1,3,15,4,2,11,6,7,12,0,5,14,9};

		static private readonly Byte[] S3 = new Byte[64] {
            10,0,9,14,6,3,15,5,1,13,12,7,11,4,2,8,
            13,7,0,9,3,4,6,10,2,8,5,14,12,11,15,1,
            13,6,4,9,8,15,3,0,11,1,2,12,5,10,14,7,
            1,10,13,0,6,9,8,7,4,15,14,3,11,5,2,12};

		static private readonly Byte[] S4 = new Byte[64] {
            7,13,14,3,0,6,9,10,1,2,8,5,11,12,4,15,
            13,8,11,5,6,15,0,3,4,7,2,12,1,10,14,9,
            10,6,9,0,12,11,7,13,15,1,3,14,5,2,8,4,
            3,15,0,6,10,1,13,8,9,4,5,11,12,7,2,14};

		static private readonly Byte[] S5 = new Byte[64] {
            2,12,4,1,7,10,11,6,8,5,3,15,13,0,14,9,
            14,11,2,12,4,7,13,1,5,0,15,10,3,9,8,6,
            4,2,1,11,10,13,7,8,15,9,12,5,6,3,0,14,
            11,8,12,7,1,14,2,13,6,15,0,9,10,4,5,3};

		static private readonly Byte[] S6 = new Byte[64] {
            12,1,10,15,9,2,6,8,0,13,3,4,14,7,5,11,
            10,15,4,2,7,12,9,5,6,1,13,14,0,11,3,8,
            9,14,15,5,2,8,12,3,7,0,4,10,1,13,11,6,
            4,3,2,12,9,5,15,10,11,14,1,7,6,0,8,13};

		static private readonly Byte[] S7 = new Byte[64] {
            4,11,2,14,15,0,8,13,3,12,9,7,5,10,6,1,
            13,0,11,7,4,9,1,10,14,3,5,12,2,15,8,6,
            1,4,11,13,12,3,7,14,10,15,6,8,0,5,9,2,
            6,11,13,8,1,4,10,7,9,5,0,15,14,2,3,12};

		static private readonly Byte[] S8 = new Byte[64] {
            13,2,8,4,6,15,11,1,10,9,3,14,5,0,12,7,
            1,15,13,8,10,3,7,4,12,5,6,11,0,14,9,2,
            7,11,4,1,9,12,14,2,0,6,10,13,15,3,5,8,
            2,1,14,7,4,10,8,13,15,12,9,0,3,5,6,11};

		static private readonly int[] ShiftTable = new int[16] { 1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1 };
		#endregion

		#region Declaration of member variables
		private Byte[] SecretKey;
		private Byte[][] RoundKey;
		private UInt32[] IntermediateData;
		#endregion

		//------------------------------------------------ Constructor
		// 鍵の最上位ビット(8, 16, ..., 64 は，パリティとする)
		public DES()
		{
			this.SecretKey = new Byte[8];
			this.RoundKey = new Byte[16][];
			this.IntermediateData = new UInt32[2];
		}

		//************************************************ Interface
		//------------------------------------------------
		public int getKeyWidth() {return 8;}
		public int getDatWidth() {return 8;}

		//------------------------------------------------ setKey()
		public void setkey(byte[] key)
		{
			for (int i = 0; i < 8; i++) this.SecretKey[i] = key[i+8];
			KeySchedule();
		}

		//------------------------------------------------ encrypt()
		public byte[] encrypt(byte[] pt)
		{
			for (int i = 0; i < 2; i++)
			{
				byte[] t = new byte[4];
				for (int j = 0; j < 4; j++) t[j] = pt[i * 4 + j];
				this.IntermediateData[i] = Utility.ByteArrayToUInt32(t);
			}

			DoEncrypt();

			byte[] ct = new byte[8];
			for (int i = 0; i < 2; i++) {
				byte[] t = Utility.UInt32ToByteArray(this.IntermediateData[i]);
				for (int j = 0; j < 4; j++) ct[i * 4 + j] = t[j];
			}
			return ct;
		}

		//------------------------------------------------ decrypt()
		public byte[] decrypt(byte[] ct) {
			for (int i = 0; i < 2; i++)
			{
				byte[] t = new byte[4];
				for (int j = 0; j < 4; j++) t[j] = ct[i * 4 + j];
				this.IntermediateData[i] = Utility.ByteArrayToUInt32(t);
			}

			DoDecrypt();

			byte[] pt = new byte[8];
			for (int i = 0; i < 2; i++) {
				byte[] t = Utility.UInt32ToByteArray(this.IntermediateData[i]);
				for (int j = 0; j < 4; j++) pt[i * 4 + j] = t[j];
			}
			return pt;
		}


		#region Functions related to key-scheduling
		public void SetKey(UInt32[] SecretKey) {
			if (SecretKey.Length != 2)
			{
				throw new System.Exception("Invalid length of argument array in SetKey");
			}
			else
			{
				for (int i = 0; i < 2; ++i)
				{
					// Conver UInt32 [] into UInt16 [].
					this.SecretKey[4 * i] = (Byte)(SecretKey[i] >> 24);
					this.SecretKey[4 * i + 1] = (Byte)((SecretKey[i] >> 16) & 0x000000ff);
					this.SecretKey[4 * i + 2] = (Byte)((SecretKey[i] >> 8) & 0x000000ff);
					this.SecretKey[4 * i + 3] = (Byte)((SecretKey[i]) & 0x000000ff);
				}
				KeySchedule();
			}
		}

		public void PrintRoundKey()
		{
			int index = 0;
			foreach (Byte[] ByteArray in RoundKey)
			{
				Console.Write("RK[{0}]:\t", index);
				Console.WriteLine(Utility.ByteArrayToString(ByteArray));
				index++;
			}
		}

		private void KeySchedule()
		{

			SecretKey = PC1(SecretKey);

			for (int i = 0; i < 16; ++i)
			{
				SecretKey = RotateBytes(SecretKey, ShiftTable[i]);
				RoundKey[i] = PC2(SecretKey);
			}
		}
		#endregion

		#region Encryption / Decryption interface
		public UInt32[] Encrypt(UInt32[] PlainText)
		{
			if (PlainText.Length != 2)
			{
				throw new System.Exception("Invalid length of argument array in Encrypt");
			}
			else
			{
				for (int i = 0; i < 2; ++i)
				{
					this.IntermediateData[i] = PlainText[i];
				}
				DoEncrypt();
			}
			UInt32[] temp = new UInt32[2];
			for (int i = 0; i < 2; ++i) temp[i] = this.IntermediateData[i];
			return temp;
		}

		private void DoEncrypt()
		{
			UInt32 FFunctionOut = 0;
			UInt32 tmp;

			IntermediateData = initPerm(IntermediateData);

			for (int i = 0; i < 16; ++i)
			{
				FFunctionOut = FFunction(IntermediateData[1], RoundKey[i]);
				tmp = IntermediateData[1];
				IntermediateData[1] = IntermediateData[0] ^ FFunctionOut;
				IntermediateData[0] = tmp;
			}
			// No crossing in the final round.
			// However the crossing is absorbed in the final permutation
			IntermediateData = finPerm(IntermediateData);
		}

		public UInt32[] Decrypt(UInt32[] CipherText)
		{
			if (CipherText.Length != 2)
			{
                throw new System.Exception("Invalid length of argument array in Encrypt");
			}
			else
			{
				// Crossing at the beginning
				IntermediateData[0] = CipherText[0];
				IntermediateData[1] = CipherText[1];
				DoDecrypt();
			}
			UInt32[] temp = new UInt32[2];
			for (int i = 0; i < 2; ++i) temp[i] = this.IntermediateData[i];
			return temp;
		}

		private void DoDecrypt()
		{
			UInt32 FFunctionOut = 0;
			UInt32 tmp;

			IntermediateData = initPerm(IntermediateData);
			for (int i = 15; i >= 0; --i)
			{
				FFunctionOut = FFunction(IntermediateData[1], RoundKey[i]);
				tmp = IntermediateData[1];
				IntermediateData[1] = IntermediateData[0] ^ FFunctionOut;
				IntermediateData[0] = tmp;
			}
			// No crossing in the final round.
			// However the crossing is absorbed in the final permutation
			IntermediateData = finPerm(IntermediateData);
		}
		#endregion

		#region DES randomize functions
		public UInt32[] initPerm(UInt32[] Data)
		{
			Byte[] Work = new Byte[8];
			Byte[] ByteArray;
			Byte Mask = (Byte)0x40;
			Byte Shift = 6;
			Byte tmp;
			UInt32[] Result;

			ByteArray = Utility.UInt32ArrayToByteArray(Data);

			for (int i = 0; i < 8; i++)
			{
				for (int j = 8; 0 < j; j = j - 1)
				{
					Work[i] = (Byte)(Work[i] << 1);
					tmp = ByteArray[j - 1];
					tmp = (Byte)(tmp & Mask);
					tmp = (Byte)(tmp >> Shift);
					Work[i] = (Byte)(Work[i] | tmp);
				}
				Mask = (Byte)(Mask >> 2);
				Shift = (Byte)(Shift - 2);
				if (Mask == 0)
				{
					Mask = 0x80;
					Shift = 7;
				}
			}
			Result = Utility.ByteArrayToUInt32Array(Work);
			return Result;
		}

		public UInt32[] finPerm(UInt32[] Data)
		{
			Byte[] ByteArray;
			Byte[] Work = new Byte[8];
			UInt32[] Result;
			Byte Mask, Shift, tmp;

			ByteArray = Utility.UInt32ArrayToByteArray(Data);

			Mask = 1;
			Shift = 0;
			for (int i = 0; i < 8; i++)
			{
				for (int j = 0; j < 4; j++)
				{
					tmp = (Byte)((ByteArray[j] & Mask) >> Shift);
					Work[i] = (Byte)(((Work[i] << 2) | (tmp << 1)
																			 | ((ByteArray[j + 4] & Mask) >> Shift)));
				}
				Mask = (Byte)(Mask << 1);
				Shift++;
			}

			Result = Utility.ByteArrayToUInt32Array(Work);
			return Result;
		}

		public Byte SboxWrapper(int SboxNum, Byte Data)
		{
			int SboxIn;
			Byte Result;
			SboxIn = (Data & 0x20) | ((Data & 0x01) << 4) | ((Data & 0x1e) >> 1);
			switch (SboxNum)
			{
				case 1: Result = S1[SboxIn]; break;
				case 2: Result = S2[SboxIn]; break;
				case 3: Result = S3[SboxIn]; break;
				case 4: Result = S4[SboxIn]; break;
				case 5: Result = S5[SboxIn]; break;
				case 6: Result = S6[SboxIn]; break;
				case 7: Result = S7[SboxIn]; break;
				case 8: Result = S8[SboxIn]; break;
				default: throw new SystemException("Invalid input to SboxWrapper");
			}
			return Result;
		}

		public UInt32 FFunction(UInt32 Data, Byte[] Key)
		{
			Byte[] tmp;
			UInt32 Result;
			tmp = EP(Data);
			for (int i = 0; i < 8; ++i)
			{
				tmp[i] ^= Key[i];
				tmp[i] = SboxWrapper(i + 1, tmp[i]);
			}
			Result = P(tmp);
			return Result;
		}

		public Byte[] EP(UInt32 x)
		{
			int i;
			Byte[] dt = new Byte[4];
			Byte[] Result = new Byte[8];
			dt[0] = (Byte)(x >> 24);
			dt[1] = (Byte)(x >> 16);
			dt[2] = (Byte)(x >> 8);
			dt[3] = (Byte)(x);

			/* 32 1 2 3 4 5 */
			Result[0] = (Byte)(((dt[3] & 0x1) << 5) | ((dt[0] & 0xf8) >> 3));

			for (i = 0; i < 3; i++)
			{
				Result[2 * i + 1] = (Byte)(((dt[i] & 0x1f) << 1) | ((dt[i + 1] & 0x80) >> 7));
				Result[2 * (i + 1)] = (Byte)(((dt[i] & 0x1) << 5) | ((dt[i + 1] & 0xf8) >> 3));
			}

			/* 28 29 30 31 32 1 */
			Result[7] = (Byte)((dt[3] & 0x1f) << 1 | ((dt[0] & 0x80) >> 7));
			return Result;
		}

		public UInt32 P(Byte[] SboxOut)
		{
			Byte[] temp = new Byte[4];
			UInt32 Result = 0;
			temp[0] = (Byte)((((SboxOut[3] & 0x1) << 7) |
																((SboxOut[1] & 0x2) << 5) |
																((SboxOut[4] & 0x1) << 5) |
																((SboxOut[5] & 0x8) << 1) |
																((SboxOut[7] & 0x8)) |
																((SboxOut[2] & 0x1) << 2) |
																((SboxOut[6] & 0x1) << 1) |
																((SboxOut[4] & 0x8) >> 3)) ^ temp[0]);
			temp[1] = (Byte)((((SboxOut[0] & 0x8) << 4) |
																((SboxOut[3] & 0x2) << 5) |
																((SboxOut[5] & 0x2) << 4) |
																((SboxOut[6] & 0x4) << 2) |
																((SboxOut[1] & 0x8)) |
																((SboxOut[4] & 0x4)) |
																((SboxOut[7] & 0x2)) |
																((SboxOut[2] & 0x4) >> 2)) ^ temp[1]);
			temp[2] = (Byte)((((SboxOut[0] & 0x4) << 5) |
																((SboxOut[1] & 0x1) << 6) |
																((SboxOut[5] & 0x1) << 5) |
																((SboxOut[3] & 0x4) << 2) |
																((SboxOut[7] & 0x1) << 3) |
																((SboxOut[6] & 0x2) << 1) |
																((SboxOut[0] & 0x2)) |
																((SboxOut[2] & 0x8) >> 3)) ^ temp[2]);
			temp[3] = (Byte)((((SboxOut[4] & 0x2) << 6) |
																((SboxOut[3] & 0x8) << 3) |
																((SboxOut[7] & 0x4) << 3) |
																((SboxOut[1] & 0x4) << 2) |
																((SboxOut[5] & 0x4) << 1) |
																((SboxOut[2] & 0x2) << 1) |
																((SboxOut[0] & 0x1) << 1) |
																((SboxOut[6] & 0x8) >> 3)) ^ temp[3]);
			Result = ((UInt32)(temp[0]) << 24) |
										 ((UInt32)(temp[1]) << 16) |
										 ((UInt32)(temp[2]) << 8) |
										 ((UInt32)(temp[3]));
			return Result;
		}

		public Byte[] PC1(Byte[] ky)
		{
			int k;
			Byte msk;
			Byte tmp;
			Byte[] work = new Byte[8];

			msk = 0x80;
			k = 7;
			for (int j = 0; j < 7; j++)
			{
				for (int i = 8; i > 0; i = i - 1)
				{
					work[j] = (Byte)(work[j] << 1);
					tmp = (Byte)(ky[i - 1] & msk);
					tmp = (Byte)(tmp >> k);
					work[j] = (Byte)(work[j] | tmp);
				}
				if (j < 3)
				{
					msk = (Byte)(msk >> 1);
					k = k - 1;
				}
				else
				{
					if (j == 3)
					{
						msk = 2;
						k = 1;
					}
					else
					{
						msk = (Byte)(msk << 1);
						k = k + 1;
					}
				}
			}
			work[7] = (Byte)((work[3] & 0x0f) << 4);
			return work;
		}

		public Byte[] PC2(Byte[] ky)
		{
			Byte[] RoundKey = new Byte[8];

			RoundKey[0] = (Byte)((((ky[1] & 0x4) << 3) |
														((ky[2] & 0x80) >> 3) |
														((ky[1] & 0x20) >> 2) |
														((ky[2] & 0x1) << 2) |
														((ky[0] & 0x80) >> 6) |
														((ky[0] & 0x08) >> 3)));
			RoundKey[1] = (Byte)((((ky[0] & 0x20)) |
														((ky[3] & 0x10)) |
														((ky[1] & 0x2) << 2) |
														((ky[0] & 0x4)) |
														((ky[2] & 0x8) >> 2) |
														((ky[1] & 0x40) >> 6)));
			RoundKey[2] = (Byte)((((ky[2] & 0x2) << 4) |
														((ky[2] & 0x20) >> 1) |
														((ky[1] & 0x10) >> 1) |
														((ky[0] & 0x10) >> 2) |
														((ky[3] & 0x40) >> 5) |
														((ky[0] & 0x1))));
			RoundKey[3] = (Byte)((((ky[1] & 0x1) << 5) |
														((ky[0] & 0x2) << 3) |
														((ky[3] & 0x20) >> 2) |
														((ky[2] & 0x10) >> 2) |
														((ky[1] & 0x8) >> 2) |
														((ky[0] & 0x40) >> 6)));
			RoundKey[4] = (Byte)((((ky[5] & 0x8) << 2) |
														((ky[6] & 0x1) << 4) |
														((ky[4] & 0x20) >> 2) |
														((ky[5] & 0x80) >> 5) |
														((ky[6] & 0x20) >> 4) |
														((ky[7] & 0x20) >> 5)));
			RoundKey[5] = (Byte)((((ky[4] & 0x40) >> 1) |
														((ky[5] & 0x10)) |
														((ky[6] & 0x2) << 2) |
														((ky[6] & 0x80) >> 5) |
														((ky[4] & 0x8) >> 2) |
														((ky[6] & 0x10) >> 4)));
			RoundKey[6] = (Byte)((((ky[5] & 0x1) << 5) |
														((ky[6] & 0x08) << 1) |
														((ky[5] & 0x20) >> 2) |
														((ky[7] & 0x10) >> 2) |
														((ky[4] & 0x4) >> 1) |
														((ky[7] & 0x80) >> 7)));
			RoundKey[7] = (Byte)((((ky[6] & 0x40) >> 1) |
														((ky[5] & 0x04) << 2) |
														((ky[6] & 0x04) << 1) |
														((ky[4] & 0x01) << 2) |
														((ky[4] & 0x80) >> 6) |
														((ky[4] & 0x10)) >> 4));
			return RoundKey;
		}
		#endregion

		public Byte[] RotateBytes(Byte[] ky, int Amount)
		{
			Byte Mask;
			int Shift;
			Byte[] tmp = new Byte[8];
			if (Amount != 1 && Amount != 2)
				throw new SystemException("Invalid Argument in");

			Shift = 8 - Amount;
			Mask = (Amount == 1) ? (Byte)0x80 : (Byte)0xc0;

			for (int i = 0; i < 7; i++)
			{
				tmp[i] = (Byte)((ky[i] << Amount) | ((ky[i + 1] & Mask) >> Shift));
			}
			if (Amount == 1)
			{
				tmp[3] = (Byte)(((ky[3] & 0x70) << 1) | ((ky[0] & Mask) >> 3));
				tmp[7] = (Byte)(((ky[7] & 0x70) << 1) | ((ky[4] & Mask) >> 3));
			}
			else
			{
				tmp[3] = (Byte)(((ky[3] & 0x30) << 2) | ((ky[0] & Mask) >> 2));
				tmp[7] = (Byte)(((ky[7] & 0x30) << 2) | ((ky[4] & Mask) >> 2));
			}
			return tmp;
		}

	}

	//================================================================ TDEA
	class TDEA
	{
		private UInt32[][] SecretKey;
		private UInt32[] IntermediateData;
		private DES DESObj;

		public TDEA()
		{
			SecretKey = new UInt32[3][];
			for (int i = 0; i < 3; ++i)
			{
				SecretKey[i] = new UInt32[2];
			}

			IntermediateData = new UInt32[2];
			DESObj = new DES();
		}

		public void SetKey(UInt32[] SecretKey)
		{
			if (SecretKey.Length != 6)
			{
				throw new System.Exception("Invalid length of argument array in SetKey");
			}
			else
			{
				for (int i = 0; i < 3; ++i)
				{
					this.SecretKey[i][0] = SecretKey[2 * i + 0];
					this.SecretKey[i][1] = SecretKey[2 * i + 1];
				}
			}
		}

		#region Encryption / Decryption interface
		public UInt32[] Encrypt(UInt32[] PlainText)
		{
			if (PlainText.Length != 2)
			{
				throw new System.Exception("Invalid length of argument array in Encrypt");
			}
			else
			{
				for (int i = 0; i < 2; ++i)
				{
					this.IntermediateData[i] = PlainText[i];
				}
				DoEncrypt();
			}
			UInt32[] temp = new UInt32[2];
			for (int i = 0; i < 2; ++i) temp[i] = this.IntermediateData[i];
			return temp;
		}

		private void DoEncrypt()
		{
			DESObj.SetKey(SecretKey[0]);
			IntermediateData = DESObj.Encrypt(IntermediateData);
			DESObj.SetKey(SecretKey[1]);
			IntermediateData = DESObj.Decrypt(IntermediateData);
			DESObj.SetKey(SecretKey[2]);
			IntermediateData = DESObj.Encrypt(IntermediateData);
		}

		public UInt32[] Decrypt(UInt32[] PlainText)
		{
			if (PlainText.Length != 2)
			{
				throw new System.Exception("Invalid length of argument array in Encrypt");
			}
			else
			{
				for (int i = 0; i < 2; ++i)
				{
					this.IntermediateData[i] = PlainText[i];
				}
				DoDecrypt();
			}
			UInt32[] temp = new UInt32[2];
			for (int i = 0; i < 2; ++i) temp[i] = this.IntermediateData[i];
			return temp;
		}

		private void DoDecrypt()
		{
			DESObj.SetKey(SecretKey[2]);
			IntermediateData = DESObj.Decrypt(IntermediateData);
			DESObj.SetKey(SecretKey[1]);
			IntermediateData = DESObj.Encrypt(IntermediateData);
			DESObj.SetKey(SecretKey[0]);
			IntermediateData = DESObj.Decrypt(IntermediateData);
		}
		#endregion
	}
}
