﻿/*-------------------------------------------------------------------------
 SASEBO Cipher Module; LSI GAIA(130nm) and CHAR(90nm)
 
 File name   : Tracer.cs
 Version     : Version 1.1
 Created     : Apr/20/2009
 Last update : Aug/18/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;

namespace SASEBO_Checker_DES
{
	//================================================================ Tracer_LSI_rev1
  class Tracer_LSI_rev1 : Tracer
  {
		//************************************************ Variable
    SASEBO.CipherModule_rev1 module;
    int num_trace;
    byte[] key;

		//************************************************ Method
		//------------------------------------------------ Constructor
		public Tracer_LSI_rev1 (SASEBO.CipherModule module, int num_trace, string key_st)
    {
      this.module    = (SASEBO.CipherModule_rev1)module;
			this.num_trace = num_trace;

      key = new byte[16];
      for (int i=0; i<8; i++) key[i] = Convert.ToByte(key_st.Substring(i * 3, 2), 16);

      worker = new BackgroundWorker();
      worker.WorkerReportsProgress      = true;
      worker.WorkerSupportsCancellation = true;
      worker.DoWork += new DoWorkEventHandler(worker_DoWork);
    }


		//************************************************ BackgroudWorker
		//------------------------------------------------ worker_DoWork()
		private void worker_DoWork(object sender, DoWorkEventArgs e)
		{
      e.Cancel = false;

      SASEBO.CipherCore core = module.cores[5];
      Cipher.CipherInterface cipher = null;
      cipher = (Cipher.CipherInterface)Activator.CreateInstance(Type.GetType("Cipher." + core.method, true));

      int progress = 0;
      e.Cancel = trace_loop_block_des(core, cipher, ref progress);
    }


		//------------------------------------------------ trace_loop_block_des()
    private bool trace_loop_block_des(SASEBO.CipherCore core, Cipher.CipherInterface cipher, ref int progress)
    {
      Random rand = new Random();

      int ksize = 16;
      int dsize = cipher.getDatWidth();
      int loop;
      bool error = false;

      byte[] text_in  = new byte[dsize];
      byte[] text_out = new byte[dsize];
      byte[] text_ans = new byte[dsize];

      // Initialization
      module.select(core.id);
      module.encdec(module.MODE_ENC);
      module.key(module.convertDESKey(key), 16); // HW
      cipher.setkey(key);                        // SW

      // start
      progress = 0;
      worker.ReportProgress(progress, (object)(new TracerReport(core.name+"...\t", 0, key, (byte[])null, text_in, text_out, text_ans)));
      for (loop = 0; loop < num_trace; loop++)
      {
        progress = (int)(100.0d * loop / num_trace);

        for (int i=0; i<dsize; i++) text_in[i] = (byte)rand.Next(256);
        // Generate answer
        text_ans = cipher.encrypt(text_in);

        // Cryptographic processing
        module.write(text_in, dsize);
        module.read(text_out, dsize);

        // check result
        for (int i=0; i < dsize; i++) if (text_out[i] != text_ans[i]) error = true;
        worker.ReportProgress(progress, (object)(new TracerReport("", loop, key, (byte[])null, text_in, text_out, text_ans)));

        // caceled ?
        if (worker.CancellationPending || error) break;
      } //endof loop

      if (worker.CancellationPending || error) error = true;

      String result;
      if (error) result = "ng.\r\n";
      else       result = "ok.\r\n";
      worker.ReportProgress(progress, (object)(new TracerReport(result, loop, key, (byte[])null, text_in, text_out, text_ans)));

      return error;
    }

  }
}
