/*-------------------------------------------------------------------------
 AES cryptographic module for FPGA on SASEBO-GII
 
 File name   : chip_sasebo_gii_aes_comp.v
 Version     : Version 1.0
 Created     : Sep/01/2009
 Last update : Sep/02/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 


//================================================ CHIP_SASEBO_GII_AES_Comp
module CHIP_SASEBO_GII_AES_Comp
  (// Local bus II
   lbus_wd, lbus_we, lbus_ful, lbus_aful,
   lbus_rd, lbus_re, lbus_emp, lbus_aemp,
   lbus_rdy, lbus_clk, lbus_rstn,
   
   trig_startn, trig_endn, trig_exec, // Trigger
   led); // LED.
   
   //------------------------------------------------
   // Local bus II
   input [7:0]  lbus_wd;
   input        lbus_we;
   output       lbus_ful, lbus_aful;
   output [7:0] lbus_rd;
   input        lbus_re;
   output       lbus_emp, lbus_aemp;
   output       lbus_rdy;
   input        lbus_clk, lbus_rstn;

   output       trig_startn, trig_endn, trig_exec; // Trigger
   output [7:0] led;           // LED
   
   //------------------------------------------------
   // Internal clock
   wire         clk, rst;

   wire [7:0]   wd, rd;
   wire         we, ful, aful, re, emp, aemp;
   wire [11:0]  rcnt;

   wire [15:0]  lbus_a, lbus_di, lbus_do;
   wire         lbus_w, lbus_r;
   
   // Block cipher
   wire [127:0] blk_kin, blk_din, blk_dout;
   wire         blk_krdy, blk_drdy, blk_kvld, blk_dvld;
   wire         blk_encdec, blk_en, blk_rstn, blk_busy;
  
   //------------------------------------------------
   assign led[0] = ~rst;
   assign led[1] = ~blk_rstn;
   assign led[2] = ~blk_encdec;
   assign led[3] = ~blk_krdy;
   assign led[4] = ~blk_kvld;
   assign led[5] = ~blk_drdy;
   assign led[6] = ~blk_dvld;
   assign led[7] = ~blk_busy;

   assign lbus_rdy = ~rst;
   
   assign trig_startn = ~blk_drdy;
   assign trig_endn   = 1'b0;//~blk_dvld;
   assign trig_exec   = 1'b0;//blk_busy;
   
   //------------------------------------------------   
   SYNCFIFO_8x2047 fifo_w
     (.wd(lbus_wd), .we(lbus_we), .ful(lbus_ful), .aful(lbus_aful),
      .rd(rd), .re(re), .emp(emp), .aemp(aemp),
      .cnt(rcnt), .clk(clk), .rst(rst));

   SYNCFIFO_8x2047 fifo_r
     (.wd(wd), .we(we), .ful(ful), .aful(aful),
      .rd(lbus_rd), .re(lbus_re), .emp(lbus_emp), .aemp(lbus_aemp),
      .cnt(), .clk(clk), .rst(rst));

   CTRL_LBUS ctrl_lbus
     (.lbus_a(lbus_a), .lbus_di(lbus_di), .lbus_do(lbus_do),
      .lbus_wr(lbus_w), .lbus_rd(lbus_r),
      .wd(wd), .we(we), .ful(ful), .aful(aful),
      .rd(rd), .re(re), .emp(emp), .aemp(aemp), .rcnt(rcnt),
      .clk(clk), .rst(rst));

   LBUS_IF lbus_if
     (.lbus_a(lbus_a), .lbus_di(lbus_di), .lbus_do(lbus_do),
      .lbus_wr(lbus_w), .lbus_rd(lbus_r),
      .blk_kin(blk_kin), .blk_din(blk_din), .blk_dout(blk_dout),
      .blk_krdy(blk_krdy), .blk_drdy(blk_drdy), 
      .blk_kvld(blk_kvld), .blk_dvld(blk_dvld),
      .blk_encdec(blk_encdec), .blk_en(blk_en), .blk_rstn(blk_rstn),
      .clk(clk), .rst(rst));

   //------------------------------------------------   
   AES_Comp U_AES_Comp
     (.Kin(blk_kin), .Din(blk_din), .Dout(blk_dout),
      .Krdy(blk_krdy), .Drdy(blk_drdy), .Kvld(blk_kvld), .Dvld(blk_dvld),
      .EncDec(blk_encdec), .EN(blk_en), .BSY(blk_busy),
      .CLK(clk), .RSTn(blk_rstn));

   //------------------------------------------------   
   MK_CLKRST mk_clkrst (.clkin(lbus_clk), .rstnin(lbus_rstn),
                        .clk(clk), .rst(rst));
endmodule // CHIP_SASEBO_GII_AES_PPRM3


   
//================================================ MK_CLKRST
module MK_CLKRST (clkin, rstnin, clk, rst);
   //synthesis attribute keep_hierarchy of MK_CLKRST is no;
   
   //------------------------------------------------
   input  clkin, rstnin;
   output clk, rst;
   
   //------------------------------------------------
   wire   refclk;
   wire   clk_dcm, locked;

   //------------------------------------------------ clock
   IBUFG u10 (.I(clkin), .O(refclk)); 
/*   
   DCM_BASE u11 (.CLKIN(~refclk), .CLKFB(clk), .RST(~rstnin),
                 .CLK0(clk_dcm),     .CLKDV(),
                 .CLK90(), .CLK180(), .CLK270(),
                 .CLK2X(), .CLK2X180(), .CLKFX(), .CLKFX180(),
                 .LOCKED(locked));
   BUFG  u12 (.I(clk_dcm),   .O(clk));
*/
   BUFG  u12 (.I(refclk),   .O(clk));

   //------------------------------------------------ reset
   MK_RST_BUFG u20 (.locked(rstnin), .clk(clk), .rst(rst));
endmodule // MK_CLKRST



//================================================ MK_RST_BUFG
module MK_RST_BUFG (locked, clk, rst);
   //synthesis attribute keep_hierarchy of MK_RST_BUFG is no;
   
   //------------------------------------------------
   input  locked, clk;
   output rst;

   //------------------------------------------------
   reg [15:0] cnt;
   wire       rst0;
   
   //------------------------------------------------
   always @(posedge clk or negedge locked) 
     if (~locked)    cnt <= 16'h0;
     else if (~&cnt) cnt <= cnt + 16'h1;

   assign rst0 = ~&cnt;
   
   BUFG u0 (.I(rst0), .O(rst));
endmodule // MK_RST_BUFG
