/*-------------------------------------------------------------------------
 SASEBO-GII controller (for FPGA cryptographic module)
 
 File name   : chip_sasebo_gii_ctrl.v
 Version     : Version 1.0
 Created     : Mar/30/2009
 Last update : Sep/02/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 


//================================================ CHIP_SASEBO_GII_CTRL
module CHIP_SASEBO_GII_CTRL
  (// FT2232D
   usb_d, usb_rdn, usb_wr, usb_rxfn, usb_txen,
   // Local bus II
   lbus_wd, lbus_we, lbus_ful, lbus_aful,
   lbus_rd, lbus_re, lbus_emp, lbus_aemp,
   lbus_rdy, lbus_clk, lbus_rstn,
   // SelectMap configuration
   cfg_din, cfg_mosi, cfg_fcsb, cfg_cclk,
   cfg_progn, cfg_initn, cfg_csn, cfg_rdwrn, cfg_busy, cfg_done, cfg_done_alt,
   // Clock, reset, and LED.
   clkin, rstnin, led);

   //------------------------------------------------
   // FT2232D
   inout [7:0]  usb_d;
   output       usb_rdn, usb_wr;
   input        usb_rxfn, usb_txen;

   // Local bus II
   output [7:0] lbus_wd;
   output       lbus_we;
   input        lbus_ful, lbus_aful;
   input [7:0]  lbus_rd;
   output       lbus_re;
   input        lbus_emp, lbus_aemp;
   input        lbus_rdy;
   output       lbus_clk, lbus_rstn;

   // Virtex-5 configuration
   input        cfg_din, cfg_mosi, cfg_fcsb, cfg_cclk;
   input        cfg_progn;
   input        cfg_csn, cfg_initn, cfg_rdwrn, cfg_busy, cfg_done, cfg_done_alt;
   
   // clock, led
   input         clkin, rstnin; // Clock and reset input
   output [7:0]  led;           // LED
   
   //------------------------------------------------
   // Internal clock
   wire         clk, rst, clkdv, rstdv;

   // Local bus
   wire [7:0] 	wd0,               wd1;
   wire 	we0, ful0, aful0,  we1, ful1, aful1;
   wire [7:0] 	rd0,               rd1;
   wire         re0, emp0, aemp0,  re1, emp1, aemp1;
      
   // etc
   reg [23:0]   cnt;
  
   //------------------------------------------------
   assign led[0] = ~cnt[23];
   assign led[1] = ~lbus_rstn;
   assign led[2] = ~(lbus_rdy & cfg_done & cfg_done_alt);
   assign led[3] = ~(cfg_initn | cfg_progn | cfg_csn | cfg_rdwrn | cfg_busy);
   assign led[4] = ~cfg_din;
   assign led[5] = ~cfg_mosi;
   assign led[6] = ~cfg_fcsb;
   assign led[7] = ~cfg_cclk;

   always @(posedge clk or posedge rst) 
     if (rst) cnt <= 24'h0;
     else     cnt <= cnt + 24'h1;

   //------------------------------------------------   
   CTRL_FT2232D ctr_ft2232d
     (.usb_d(usb_d), .usb_rdn(usb_rdn), .usb_wr(usb_wr),
      .usb_rxfn(usb_rxfn), .usb_txen(usb_txen),
      .wd(wd0), .we(we0), .ful(ful0), .aful(aful0),
      .rd(rd0), .re(re0), .emp(emp0), .aemp(aemp0),
      .lbus_rdy(lbus_rdy & cfg_done & cfg_done_alt),
      .clk(clk), .rst(rst));

   //------------------------------------------------ lbus_wr
   ASYNCFIFO_8x15 afifo_wr
     (.wd(wd0), .we(we0), .ful(ful0), .aful(aful0), .wclk(clk), .wrst(rst),
      .rd(rd1), .re(re1), .emp(emp1), .aemp(aemp1), .rclk(clkdv), .rrst(rstdv));

   BUS_CONNECTOR bus_wr 
     (.wd(lbus_wd), .we(lbus_we), .ful(lbus_ful), .aful(lbus_aful),
      .rd(rd1), .re(re1), .emp(emp1), .aemp(aemp1),
      .clk(clkdv), .rst(rstdv));

   //------------------------------------------------ lbus_rd
   ASYNCFIFO_8x15 afifo_rd
     (.wd(wd1), .we(we1), .ful(ful1), .aful(aful1), .wclk(clkdv), .wrst(rstdv),
      .rd(rd0), .re(re0), .emp(emp0), .aemp(aemp0), .rclk(clk), .rrst(rst));

   BUS_CONNECTOR bus_rd
     (.wd(wd1), .we(we1), .ful(ful1), .aful(aful1),
      .rd(lbus_rd), .re(lbus_re), .emp(lbus_emp), .aemp(lbus_aemp),
      .clk(clkdv), .rst(rstdv));
	//synthesis attribute KEEP of lbus_aemp is true;
   
   //------------------------------------------------   
   MK_CLKRST mk_clkrst 
     (.clkin(clkin), .rstnin(rstnin), .cfg_done(cfg_done & cfg_done_alt),
      .clk(clk), .rst(rst), .clkdv(clkdv), .rstdv(rstdv));

   assign lbus_clk  = ~clkdv;
   assign lbus_rstn = ~rstdv;
   
endmodule // CHIP_SASEBO_GII_CTRL



//================================================ MK_CLKRST
module MK_CLKRST (clkin, rstnin, cfg_done, clk, rst, clkdv, rstdv);
   //synthesis attribute keep_hierarchy of MK_CLKRST is no;
   
   //------------------------------------------------
   input  clkin, rstnin, cfg_done;
   output clk, rst, clkdv, rstdv;
   
   //------------------------------------------------
   wire   rst_dll;
   wire   refclk;
   wire   clk_dcm, clkdv_dcm, locked;
   wire   rstn;

   //------------------------------------------------ dll reset
   MASTER_RST u00 (.clk(refclk), .rst(rst_dll));

   //------------------------------------------------ clock
   IBUFG u10 (.I(clkin), .O(refclk)); 

   DCM   u11 (.CLKIN(refclk), .CLKFB(clk), .RST(rst_dll),
              .PSEN(1'b0), .PSINCDEC (1'b0), .PSCLK(1'b0), .DSSEN(1'b0),
              .CLK0(clk_dcm), .CLKDV(clkdv_dcm),
              .CLK90(), .CLK180(), .CLK270(),
              .CLK2X(), .CLK2X180(), .CLKFX(), .CLKFX180(),
              .STATUS(), .LOCKED(locked), .PSDONE());
   //synthesis attribute CLKDV_DIVIDE of u11 is 12.0;
   defparam u11.CLKDV_DIVIDE = 12.0;
 
   BUFG  u12 (.I(clk_dcm),   .O(clk));
   BUFG  u13 (.I(clkdv_dcm), .O(clkdv));

   //------------------------------------------------ reset
   assign rstn = locked & rstnin & cfg_done;
   MK_RST_BUFG u20 (.locked(rstn), .clk(clk), .rst(rst));
   SRL16 u21 (.D(rst), .CLK(clkdv), .Q(rstdv),
	      .A3(1'b1), .A2(1'b1), .A1(1'b1), .A0(1'b1));
   defparam u21.INIT = 16'hFFFF;
   //synthesis attribute INIT of u21 is 16'hFFFF;

endmodule // MK_CLKRST



//================================================ MASTER_RST
module MASTER_RST (clk, rst);
   //synthesis attribute keep_hierarchy of MASTER_RST is no;

   //------------------------------------------------
   input  clk;
   output rst;

   //------------------------------------------------
   wire   rst0;
   
   //------------------------------------------------
   SRL16 u0 (.D(1'b0), .Q(rst0), .CLK(clk),
	     .A3(1'b1), .A2(1'b1), .A1(1'b1), .A0(1'b1));
   defparam u0.INIT = 16'hFFFF;
   //synthesis attribute INIT of u0 is 16'hFFFF;

   SRL16 u1 (.D(rst0), .Q(rst),  .CLK(clk),
	     .A3(1'b1), .A2(1'b1), .A1(1'b1), .A0(1'b1));
   defparam u1.INIT = 16'hFFFF;
   //synthesis attribute INIT of u1 is 16'hFFFF;
   
endmodule // MASTER_RST



//================================================ MK_RST_BUFG
module MK_RST_BUFG (locked, clk, rst);
   //synthesis attribute keep_hierarchy of MK_RST_BUFG is no;
   
   //------------------------------------------------
   input  locked, clk;
   output rst;

   //------------------------------------------------
   reg [23:0] cnt;
   wire       rst0;
   
   //------------------------------------------------
   always @(posedge clk or negedge locked) 
     if (~locked)    cnt <= 24'h0;
     else if (~&cnt) cnt <= cnt + 24'h1;

   assign rst0 = ~&cnt;
   BUFG u0 (.I(rst0), .O(rst));
endmodule // MK_RST_BUFG
