
//================================================ FIFO_CONNECTOR_8
module FIFO_CONNECTOR_8 
  (rd, re, emp, aemp, 
   wd, we, ful, aful, 
   clk, rst);
   
   //------------------------------------------------
   parameter WIDTH = 8;
   input [WIDTH-1:0]  rd;
   output 	      re;
   input 	      emp, aemp;
   output [WIDTH-1:0] wd;
   output 	      we;
   input 	      ful, aful;
   input 	      clk, rst;
   
   //------------------------------------------------
   wire 	      rd_busy, wr_busy;
   reg 		      re, re1;
   reg [WIDTH-1:0]    wd0, wd1, wd2;
   reg 		      we0, we1, we2;

   //------------------------------------------------
   assign rd_busy = emp | (aemp & re);
   assign wr_busy = ful | (aful & we);

   always @(posedge clk)
     if (rst)          re <= 0;
     else if (rd_busy) re <= 0;
     else if (wr_busy) re <= 0;
     else if (re)      re <= 0;
     else              re <= 1;

   always @(posedge clk)
     if (rst) re1 <= 0;
     else     re1 <= re;

   //------------------------------------------------
   assign wd = wd0;
   assign we = we0;

   //------------------------------------------------
   always @(posedge clk)
     if (rst)        begin wd0 <= 0;   we0 <= 0; end
     else if (~wr_busy) 
       if (we1)      begin wd0 <= wd1; we0 <= 1; end
       else if (re1) begin wd0 <= rd;  we0 <= 1; end
       else          begin             we0 <= 0; end 
     else            begin             we0 <= 0; end

   //------------------------------------------------
   always @(posedge clk)
     if (rst)              begin wd1 <= 0; we1 <= 0; end
     else if (~wr_busy) 
       if (we2)            begin wd1 <= wd2; we1 <= 1; end
       else if (re1 & we1) begin wd1 <= rd;  we1 <= 1; end
       else                begin             we1 <= 0; end
     else
       if (re1 & ~we1)     begin wd1 <= rd;  we1 <= 1; end

   //------------------------------------------------
   always @(posedge clk)
     if (rst)            begin wd2 <= 0;  we2 <= 0; end
     else if (~wr_busy)  begin	          we2 <= 0; end
     else if (re1 & we1) begin wd2 <= rd; we2 <= 1; end

endmodule // FIFO_CONNECTOR_8
